;+
; NAME:
;       SCALE2SWB_MLSO
;
; PURPOSE:
;       This function scales any input solar disk image to the default FOV of 
;       the Solar Weather Browser, as it is defined in !swbs.
;
; CATEGORY:
;       SWB, gen      
;
; CALLING SEQUENCE:
;       Result = SCALE2SWB_MLSO (Xcenter, Ycenter, Solradius, Data)
;
; INPUTS:
;       Xcenter: Solar center position (x axis) in pixels as given in FITS header 
;       Ycenter: Solar center position (y axis) in pixels as given in FITS header
;       Solradius: Solar Radius in pixel as given in FITS heacer
;       Data: Solar image to scale (2d array)
;       
; OUTPUTS:
;       This function returns an image scaled to the SWB fov.
;
; MODIFICATION HISTORY:
; Written by:    C. Marque, 11 jan. 2007
; 01 Oct 2007 [ALS]: Use set_swbs_mlso for MLSO data.
;-

FUNCTION scale2swb_mlso, xcenter, ycenter, solradius, data

set_swbs_mlso

xsd = (size (data))[1]
ysd = (size (data))[2]

; ------------------------------------------------------------------------------
; First put the center of the Sun at the center of the array.
; ------------------------------------------------------------------------------

scf     = xsd / 512.0
xoffset = xcenter - xsd / 2
yoffset = ycenter - ysd / 2
datai   = interpolate (data, findgen (xsd) + xoffset, findgen (ysd) + yoffset, $
                      /grid, missing = min (data))

; ------------------------------------------------------------------------------
; Then rebin on a 512x512 grid with a FOV fixed by !swbs and re-center the map>.
; ------------------------------------------------------------------------------

swbf = solradius / scf / !swbs.ref_solradius

;dataii = interpolate (datai, findgen (512) * scf * swbf, #
;                       findgen (512) * scf * swbf, /grid, cubic = -0.5, $
;                       missing = min (datai))

dataii = interpolate (datai, (findgen (512) + (256/swbf-256)) * scf * swbf, $
                             (findgen (512) + (256/swbf-256)) * scf * swbf, $
                             /grid, missing = min (datai))

; ------------------------------------------------------------------------------
; Then re-center the map.
; ------------------------------------------------------------------------------

;dataii = interpolate(temporary(dataii),findgen(512)+(256/swbf-256),findgen(512)+(256/swbf-256),/grid,cubic = -0.5,missing = min(datai))

datai = 0.0
RETURN, dataii
END
