;======================== header ===============================================
;+
; NAME:
;   picsfswb
; PURPOSE:
;   Make JPEG files for Solar Weather Browser:
;   MLSO PICS (H-alpha Disc Flat) images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   date: yyyymmdd
;   /logfile:	if set, print diagnostics to a logfile (default=stdout=screen)
;   /new:	if set, process ONLY "new" images (not previously done).
;   /view:	if set, view the processed images in an IDL window.
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger [ALS], ROB, 04 Jun 2007, from "mk42zip.pro".
; 07 Jun 2007 [ALS]; Add rotation to correct for solar P-angle.
; 21 Jun 2007 [ALS]: Set in/out directories for HAO/MLSO archive.
; 29 Jun 2007 [ALS]: Set pixels in outer FOV to zero.
; 25 Jul 2007 [ALS]: Add "view" keyword to display image in IDL window.
; 30 Jul 2007 [ALS]: Use scale2swb to resize image.
; 02 Aug 2007 [ALS]: Rename from "picsf2zip.pro" to "picsf2swb.pro".
; 10 Aug 2007 [ALS]: Remove "hdr" parameter in write2zip.
; 18 Sep 2007 [ALS]: Rename from "picsf2swb" to "picsfswb".  Add "new" keyword.
;  1 Oct 2007 [ALS]: Use "scale2swb_mlso.pro".
;-
;======================== end header ===========================================

PRO picsfswb, logfile=logfile, date=date, view=view, new=new

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'picsfswb'

; ------------------------------------------------------------------------------
; Define SWB reference parameters.
; ------------------------------------------------------------------------------

set_swbs_mlso

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

print, 'outdim: ', outdim
print, 'ref_soladius: ', ref_solradius
print, 'ref_xcenter:  ', ref_xcenter
print, 'ref_ycenter:  ', ref_ycenter

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

IF (keyword_set (date))  THEN	$
BEGIN
    year  = strmid (date, 0, 4)
    month = strmid (date, 4, 2)
    day   = strmid (date, 6, 2)
END	$
ELSE	$
BEGIN
   print, "No date specified."
   RETURN
END

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

;datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'
datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'

print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; If "view" keyword is set, open IDL window.
; ------------------------------------------------------------------------------

if (keyword_set (view))  THEN	$
    window, 2, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Create list of images.
; ------------------------------------------------------------------------------

sdir = datadir + '*dpm.asl.fts*'
print, 'sdir: ', sdir
;spawn, 'ls ' + sdir, list

list = file_search (sdir)

; ------------------------------------------------------------------------------
; Image processing loop.
; ------------------------------------------------------------------------------

IF (list [0] EQ '') THEN $
BEGIN ;{ ---------- Empty List ----------
    print,       '*** No images available for: ', datadir
    printf, log, '*** No images available for: ', datadir
END $ ;}

ELSE $
BEGIN ;{ ------- List not empty -------------

    IF (KEYWORD_SET (new))  THEN	$
    BEGIN ;{ ---------- Process only new images ----------

	; ----------------------------------------------------------------------
	; Create a list of previously-processed image times for specified date.
	; ----------------------------------------------------------------------

        plist = swb_readlist (date, stlf='picsf', odir='/archive/swbs/mlso/')
	plist = STRMID (plist, STRPOS (plist [0], '.jpg') - 15, 15)

	; ----------------------------------------------------------------------
	; Create a list of FITS image times to be processed.
	; ----------------------------------------------------------------------

	dlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 1, 8)
	tlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 10, 6)
	ilist = dlist + '_' + tlist

	; ----------------------------------------------------------------------
	; Compare the two lists.
	; ----------------------------------------------------------------------

	nlist = cmset_op (ilist, 'AND', /not2, plist, /index)

;	print, 'dlist: '
;	print,  dlist
;	print, 'tlist: '
;	print,  tlist
;	print, 'size (ilist): ', size (ilist)

	print, 'ilist: '
	print,  ilist

	print, 'plist: '
	print,  plist

	print, 'nlist: '
	print,  nlist

	; ----------------------------------------------------------------------
	; Create a list of images which have NOT been processed previously.
	; If the list is empty, there is nothing to do for this day.
	; ----------------------------------------------------------------------

	if (nlist[0] NE -1) THEN	$
	    list = list [nlist]		$
	ELSE				$
	    BEGIN
		print,       '*** No new images to process for: ', datadir
		printf, log, '*** No new images to process for: ', datadir
	        goto, end_of_process
	    END				
    END ;}

    ; --------------------------------------------------------------------------
    ; Image processing loop.
    ; --------------------------------------------------------------------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
;    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report image being proccessed.
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'input  image ', list[i]
    print,       'input  image ', list[i]

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "picsf_zip: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    pics_date = index.date_obs
    pics_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (pics_date,  0, 4)
    month    = strmid (pics_date,  5, 2)
    day      = strmid (pics_date,  8, 2)

    hours    = strmid (pics_time,  0, 2)
    mins     = strmid (pics_time,  3, 2)
    sec      = strmid (pics_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Rotation angle & sun center.
    ; --------------------------------------------------------------------------

    rot_ang = -index.crota1		; rotation angle.
    xcen    = index.crpix1		; x-axis center.
    ycen    = index.crpix2		; y-axis center.

    print, 'xcen, ycen : ', xcen, ycen

    ; --------------------------------------------------------------------------
    ; Rotate image to correct for the P-angle.
    ; --------------------------------------------------------------------------

    IF (index.crota1 NE 0.0)  THEN	$
    BEGIN
        img = rot (img, rot_ang, 1.0, xcen, ycen,  missing=0, /interp)
        print, 'Rotate image ... rot_ang : ', rot_ang
    END

    ; --------------------------------------------------------------------------
    ; Crop image to square shape [1024x1024].
    ; --------------------------------------------------------------------------

    xdim   = index.naxis1
    ydim   = index.naxis2

    x1 = (xdim - 1024) / 2
    x2 = x1 + 1023
    y1 = (ydim - 1024) / 2
    y2 = y1 + 1023

    print, 'xdim, ydim: ', xdim, ydim
    print, 'x1, x2, y1, y2: ', x1, x2, y1, y2

    IF (xdim GE 1024 AND ydim GE 1024) THEN	$
    BEGIN
        imgsq = img [x1:x2, y1:y2] 			; Crop image: 1024x1024
    END					$
    ELSE					$
    BEGIN
        print, 'invalid image size: ', xdim, ydim
        RETURN
    END

    ; --------------------------------------------------------------------------
    ; Square image parameters.
    ; --------------------------------------------------------------------------

    size_img = size (imgsq)			; size of croped image.
    sxdim = size_img [1]
    sydim = size_img [2]

    sxcen  = (index.crpix1 - x1)		; x-axis sun center: square.
    sycen  = (index.crpix2 - y1)		; y-axis sun center: square.
    pixrs     = index.crradius		; pixels/Rsun

    print, 'sxdim, sydim: ', sxdim, sydim
    print, 'pixrs: ', pixrs

    ; --------------------------------------------------------------------------
    ; Re-size image to SWB reference resolution.
    ; --------------------------------------------------------------------------

    imgswb    = scale2swb_mlso (sxcen, sycen, pixrs, imgsq)	; Re-size image.

    fov_ratio = 1.0

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin				; display minimum.
    dispmax = index.dispmax				; display maximum.

    imgswb  = imgswb - 4				; Subtract bkg level.
    mask_im = where (imgswb LE min (imgswb))		; Clip dim values.
    imgswb [mask_im] = 0  
    imgswb    = imgswb > 0		; Eliminate negative pixel values.

    imin = min (imgswb, max=imax)			; Get min/max.

    print, 'imin, imax: ', imin, imax

    imin =  700
    imax = 1200

    print, 'imin, imax: ', imin, imax

    imgswb = bytscl (imgswb, min=imin, max=imax)	; Re-scale brightness.

    imin = min (imgswb, max=imax)			; Get min/max.
    print, 'imin, imax: ', imin, imax, ' [after scaling]'

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

;    imgswb = smooth (img, 3)

    ; --------------------------------------------------------------------------
    ; Set pixels in outer FOV to zero.
    ; --------------------------------------------------------------------------

    icolor =   0 
    imgswb = disc_mask (imgswb, icolor, outdim, outdim, $
                        ref_xcenter, ref_ycenter, ref_solradius)

    print, 'Mask off outer FOV [corners].'

    ; --------------------------------------------------------------------------
    ; Display image (if "view" keyword is set).
    ; --------------------------------------------------------------------------

    IF (keyword_set (view))  THEN	$
        tv, imgswb

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun		; arcsec/Rsun.

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

    pics_fov = sxdim * index.cdelt1	; arcsec in PICS FOV.

;    fov_ratio = pics_fov / swb_fov

;    print, 'cdelt1:   ', index.cdelt1
;    print, 'swb_fov: ', swb_fov, '  pics_fov: ', pics_fov, $
;           '  fov_ratio: ', fov_ratio

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn = 'mlso_picsf_' + year + month + day + '_' + hours + mins + sec
    outdir = !swbs.out_dir + year + '/' + month + '/' + day + '/'

;    outdir = '/archive/swbs/' + 'mlso/' + year + '/' + month + '/' + day + '/'

    hdr              = ''
    data_type        = 'I'
    data_name        = 'MLSO H-alpha Disc Flat'
    color_table_code = '001' 				; greyscale.

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2zip, outfn, outdir, imgswb, data_type, data_name, $
               color_table_code, date, /logfile

    printf, log, 'output image ', outdir + outfn
    print,       'output image ', outdir + outfn
    print,       ''

    ; --------------------------------------------------------------------------

    ENDFOR ;} i
ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Last log message.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, ' <<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

end_of_process:
IF (keyword_set (logfile)) THEN free_lun, log   
END
