;+
; NAME		corona_mask.pro
;
; PURPOSE	Set occulting disc and outer FOV pixels to zero.
;
; SYNTAX	corona_mask, img, icolor, xdim, ydim, xcen, ycen, pixrs,
;                            rmin=rmin, rmax=rmax
;
;		img		image array
;		icolor		color index (0-255) to set pixels in mask area.
;		xdim, ydim	image dimensions.
;		xcen, ycen	sun center.
;		pixrs		# pixels per solar radius.
;		rmin, rmax	inner and out FOV (Rsun units).
;
; EXAMPLES	img = corona_mask (img, icolor, xdim, ydim, xcen, yxcen, pixrs,$
;                                  rmin=rmin, rmax=rmax)
;
; EXTERNAL
; PROCEDURES	none
;
; HISTORY	Andrew L. Stanger   Royal Observatory of Belgium   29 Jun 2007
;-

FUNCTION corona_mask, img, icolor, xdim, ydim, xcen, ycen, pixrs, $
                      rmin=rmin, rmax=rmax

   ; ---------------------------------------------------------------------------
   ; Default min and max radii.
   ; ---------------------------------------------------------------------------

   radmin = 1.00

   ; ---------------------------------------------------------------------------
   ; Compute distance from sun center to right edge of image..
   ; ---------------------------------------------------------------------------

   xdist = float (xdim-1) - xcen		; radial distance FOV (pixels).
   xmax  = float (xdist) / pixrs		; radial distance FOV (Rsun).
   radmax = xmax

   ydist = float (ydim-1) - ycen		; radial distance FOV (pixels).
   ymax  = float (ydist) / pixrs		; radial distance FOV (Rsun).
   radmax = ymax

   IF (NOT keyword_set (rmin))  THEN rmin = radmin
   IF (NOT keyword_set (rmax))  THEN rmax = radmax

   print, 'xdist, ydist: ', xdist, ydist

   print, 'corona_mask> xdim, ydim: ', xdim, ydim
   print, 'corona_mask> xcen, ycen: ', xcen, ycen
   print, 'corona_mask> pixrs:      ', pixrs
   print, 'corona_mask> radmin, radmax: ', radmin, radmax

   ; ---------------------------------------------------------------------------
   ; Set pixels inside rmin and outside rmax to the color "icolor".
   ; ---------------------------------------------------------------------------

   FOR iy = 0, ydim - 1 DO $
   BEGIN
      y = float (iy) - ycen
      FOR ix = 0, xdim - 1 DO $
      BEGIN
	 x = float (ix) - xcen
	 d = sqrt (x*X + y*y)				; pixel units
	 r = d / pixrs			 		; solar radius units.
         IF (r LT rmin) THEN img [ix, iy] = icolor	; inner FOV limit.
         IF (r GT rmax) THEN img [ix, iy] = icolor	; outer FOV limit.
      END
   END

RETURN, img
END
